import 'package:flutter/material.dart';

mixin _WingsPluginLifecycle {
  Future<void> beforeRunApp();

  Function onRunApp(Function fun);

  Widget onCreateApp(Widget widget);

  Future<void> onPage();
}

abstract class WingsPlugins with _WingsPluginLifecycle {
  List<WingsPlugin> get plugins;

  @override
  beforeRunApp() async {
    (plugins ?? []).forEach((e) async {
      await e.beforeRunApp();
    });
  }

  @override
  Function onRunApp(Function fun) {
    Function f = fun;
    (plugins ?? []).forEach((e) {
      f = (e.onRunApp(f));
    });
    f();
    return null;
  }

  @override
  Widget onCreateApp(Widget widget) {
    Widget w = widget;
    (plugins ?? []).forEach((e) {
      w = e.onCreateApp(w);
    });
    return w;
  }

  @override
  onPage() async {
    (plugins ?? []).forEach((e) async {
      await e.onPage();
    });
  }
}

abstract class WingsPlugin with _WingsPluginLifecycle {
  @override
  beforeRunApp() async {}

  @override
  Function onRunApp(Function fun) {
    return fun;
  }

  @override
  Widget onCreateApp(Widget widget) {
    return widget;
  }

  @override
  onPage() async {}
}
